
//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
//-----------------------------------------------------------------------------
// EFM8LB1_ADC_Autoscan_Large_Buffer.c
//-----------------------------------------------------------------------------
// Copyright 2015 Silicon Laboratories, Inc.
// http://developer.silabs.com/legal/version/v11/Silicon_Labs_Software_License_Agreement.txt
//
// Program Description:
//
// This program demonstrates the use of the ADC's Autoscan feature to fill a
// 2048 sample buffer using the EFM8 ADC peripheral driver library.
//
// The ADC's Autoscan feature can be used to fill a buffer with up to 64 samples
// without any CPU intervention. In this example, a single 2048 sample buffer is
// successively filled, 64 samples at a time, using autoscan mode.
//
// Timer 2 is set to trigger ADC conversions at 64 Hz. Each conversion fills one
// place in the current buffer. After the buffer is full (one second of samples),
// the ADC Conversion Complete callback is called, allowing the next buffer to be
// set as the active buffer.
//
// Inside the main loop, the CPU waits in Idle mode until an interrupt is triggered.
// This occurs once an autoscan is complete. After this, the full buffer's contents
// are printed out through UART, while the next buffer is loaded as the next
// autoscan buffer.
//
//
//
//-----------------------------------------------------------------------------
// Target:         EFM8LB1
// Tool chain:     Generic
//
// Release 0.1 (BL)
//    - Initial Revision
//    - 04 AUG 2015
//

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <SI_EFM8LB1_Register_Enums.h>
#include "InitDevice.h"
#include "adc_0.h"
#include "stdio.h"
#include "stdlib.h"
#include <math.h>
#include "EFM8LB1_FlashPrimitives.h"
#include "EFM8LB1_FlashUtils.h"
//#include "bsp.h"

//-----------------------------------------------------------------------------
#define START_ADDRESS_B          0x1A00 //  . 0
#define START_ADDRESS_K          0x1600 //  . 1
#define START_ADDRESS_P          0x1200 //  

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------
SI_LOCATED_VARIABLE_NO_INIT(flash_write_array[512], uint8_t, const SI_SEG_CODE, START_ADDRESS_B);
SI_LOCATED_VARIABLE_NO_INIT(flash_write_array1[512], uint8_t, const SI_SEG_CODE, START_ADDRESS_K);
SI_LOCATED_VARIABLE_NO_INIT(flash_write_array2[512], uint8_t, const SI_SEG_CODE, START_ADDRESS_P);
//-----------------------------------------------------------------------------
// Autoscan buffer initialization
//-----------------------------------------------------------------------------
#define BUFFER_SIZE (1024)      // 2048-2048 The size, in samples, of the autoscan
                                // buffer BUFFER
#define SAMPLES_PER_SCAN (64)
// The starting address of autoscan buffer BUFFER in XDATA space.
// MUST END ON AN EVEN ADDRESS!
#define BUFFER_START_ADDR (0x0000)
ADC0_NEW_AUTOSCAN_BUFFER_ARRAY(BUFFER, BUFFER_SIZE, BUFFER_START_ADDR);
//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------
//ADC0_AutoscanBuffer_t * CURRENT_BUFFER;   // A pointer to an autoscan buffer
//bool CONVERSION_COMPLETE = false;
//-----------------------------------------------------------------------------
// Function Prototypes
//-----------------------------------------------------------------------------
void SUMMA(void);
void readADC_XRAM();
void StartAutoscan(void);
//----------------
void DEL2US() {
	unsigned char i;
	for (i = 0; i < 100; i++) {
	}
}
//-----------
void DEL20NS() {
	unsigned char i;
	for (i = 0; i < 1; i) {i++;}
}
//-----------
void DEL3MS() {
	unsigned short i;
	for (i = 0; i < 60000; i++) {
	}
}
void DEL30MS() {
	unsigned char i;
	for (i = 0; i < 10; i++) {
	DEL3MS();
	}
}
//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------
bool BUFFER_FULL = false;
uint16_t BUFFER_INDEX = 0;

//-------------------------------------
union {                      // AD0.UL - 4-  ulong 
   unsigned char BT[4];      // AD0.BT[0]-  ..
   unsigned long UL;         // AD0.BT[1]- ..
   }AD0;                     // AD0.BT[2]- ..
                             // AD0.BT[3]- ..
//----------
   union {                   // KWR.F - 4-  float  -  K  .
      unsigned char B[4];    // KWR.B[0]-  ..
      float F;               // KWR.B[1]- ..
      }KWR;                  // KWR.B[2]- ..
                             // KWR.B[3]- ..
//--------------------------------------------------------
    union {                  // KRD.F - 4-  float  -  K  .
     unsigned char B[4];     // KRD.B[0]-  ..
     float F;                // KRD.B[1]- ..
      }KRD;                  // KRD.B[2]- ..
                             // KRD.B[3]- ..
 //--------------------------------------------------------
     union {                 // BWR.F - 4-  float  -  B  .
     unsigned char B[4];     // BWR.B[0]-  ..
     float F;                // BWR.B[1]- ..
         }BWR;               // BWR.B[2]- ..
                             // BWR.B[3]- ..
  //--------------------------------------------------------
     union {                 // BRD.F - 4-  float  -  B  
     unsigned char B[4];     // BRD.B[0]-  ..
     float F;                // BRD.B[1]- ..
         }BRD;               // BRD.B[2]- ..
                             // BRD.B[3]- ..
  //--------------------------------------------------------
      union {                // PWR.F - 4-  float  -  P  .
      unsigned char B[4];    // PWR.B[0]-  ..
      float F;               // PWR.B[1]- ..
          }PWR;              // PWR.B[2]- ..
                             // PWR.B[3]- ..
  //--------------------------------------------------------
      union {                 // PRD.F - 4-  float  -  P  
      unsigned char B[4];     // PRD.B[0]-  ..
             float F;         // PRD.B[1]- ..
          }PRD;               // PRD.B[2]- ..
                              // PRD.B[3]- ..
  //--------------------------------------------------------

   unsigned char c,j,jk,sdv,l,k,dt,INB,N,point,buf1[4],buf2[4],buf3[10],MB0,ml2,DIA,IC;
   //unsigned short i,dt,tn;
   unsigned short i,tn,psta,pmla;
   float F,koef,koefsr;
   //-------------

   //unsigned long proba;

//-------------------------
   sbit DIAP = P1 ^ 2;//DIAP - : 6  60.

   sbit CS = P1 ^ 6;//CS
   sbit DC = P2 ^ 0;//DC
   sbit RES = P2 ^ 1;//RES

   sbit CAL0 = P2 ^ 2;//CAL0 -  .
   sbit CAL1 = P2 ^ 3;//CAL1 -    (3).
   //----------------------
   unsigned char M[10] = {
//  "0"  "1"  "2"  "3"  "4"  "5"  "6"  "7"  "8"  "9"
	0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x07,0x7f,0x6f
   };
//-----------------------------------
   unsigned char MD [12] [72] = {//Arial_Narrow18x32
           {
           0x00, 0x00, 0x00, 0x00,
           0x00, 0xFE, 0x7F, 0x00,
           0xE0, 0xFF, 0xFF, 0x07,
           0xF8, 0xFF, 0xFF, 0x1F,
           0xFC, 0xFF, 0xFF, 0x3F,
           0xFE, 0xFF, 0xFF, 0x7F,
           0xFE, 0x00, 0x00, 0x7F,
           0x3F, 0x00, 0x00, 0xFC,
           0x1F, 0x00, 0x00, 0xF8,
           0x1F, 0x00, 0x00, 0xF8,
           0x1F, 0x00, 0x00, 0xF8,
           0x3F, 0x00, 0x00, 0xFC,
           0xFE, 0x00, 0x00, 0x7F,
           0xFE, 0xFF, 0xFF, 0x7F,
           0xFC, 0xFF, 0xFF, 0x3F,
           0xF8, 0xFF, 0xFF, 0x1F,
           0xE0, 0xFF, 0xFF, 0x07,
           0x00, 0xFE, 0x7F, 0x00  // Code for char 0
           },
           {
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x3E, 0x00, 0x00,
           0x00, 0x1F, 0x00, 0x00,
           0x80, 0x1F, 0x00, 0x00,
           0x80, 0x0F, 0x00, 0x00,
           0xC0, 0x07, 0x00, 0x00,
           0xF0, 0x07, 0x00, 0x00,
           0xF8, 0x03, 0x00, 0x00,
           0xFE, 0xFF, 0xFF, 0xFF,
           0xFF, 0xFF, 0xFF, 0xFF,
           0xFF, 0xFF, 0xFF, 0xFF,
           0xFF, 0xFF, 0xFF, 0xFF,
           0xFF, 0xFF, 0xFF, 0xFF,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00  // Code for char 1
           },
           {
           0x00, 0x00, 0x00, 0x00,
           0xC0, 0x01, 0x00, 0xF8,
           0xF0, 0x01, 0x00, 0xFE,
           0xFC, 0x01, 0x80, 0xFF,
           0xFC, 0x01, 0xC0, 0xFF,
           0xFE, 0x01, 0xE0, 0xFF,
           0x7F, 0x00, 0xF8, 0xFF,
           0x3F, 0x00, 0xFC, 0xFF,
           0x1F, 0x00, 0xFE, 0xF9,
           0x1F, 0x00, 0xFF, 0xF8,
           0x1F, 0x80, 0x7F, 0xF8,
           0x3F, 0xC0, 0x3F, 0xF8,
           0x7F, 0xF0, 0x1F, 0xF8,
           0xFE, 0xFF, 0x0F, 0xF8,
           0xFE, 0xFF, 0x07, 0xF8,
           0xFC, 0xFF, 0x01, 0xF8,
           0xF0, 0x7F, 0x00, 0xF8,
           0xC0, 0x1F, 0x00, 0x00,  // Code for char 2
           },
           {
           0x00, 0x00, 0x00, 0x00,
           0xC0, 0x00, 0x80, 0x03,
           0xF0, 0x00, 0x80, 0x0F,
           0xFC, 0x01, 0xC0, 0x1F,
           0xFE, 0x01, 0xC0, 0x3F,
           0xFE, 0x01, 0xC0, 0x7F,
           0x7F, 0x00, 0x00, 0x7F,
           0x3F, 0xF0, 0x01, 0xFC,
           0x1F, 0xF0, 0x01, 0xF8,
           0x1F, 0xF0, 0x01, 0xF8,
           0x1F, 0xF8, 0x01, 0xF8,
           0x3F, 0xFC, 0x03, 0xFC,
           0xFE, 0xFF, 0x07, 0x7E,
           0xFE, 0xBF, 0xFF, 0x7F,
           0xFC, 0xBF, 0xFF, 0x3F,
           0xF8, 0x0F, 0xFF, 0x1F,
           0xE0, 0x07, 0xFE, 0x0F,
           0x00, 0x00, 0xF8, 0x03  // Code for char 3
           },
           {
           0x00, 0x00, 0xF0, 0x03,
           0x00, 0x00, 0xFC, 0x03,
           0x00, 0x00, 0xFE, 0x03,
           0x00, 0x80, 0xFF, 0x03,
           0x00, 0xE0, 0xFF, 0x03,
           0x00, 0xF8, 0xEF, 0x03,
           0x00, 0xFE, 0xE3, 0x03,
           0x80, 0xFF, 0xE0, 0x03,
           0xC0, 0x3F, 0xE0, 0x03,
           0xF0, 0x0F, 0xE0, 0x03,
           0xFC, 0x07, 0xE0, 0x03,
           0xFF, 0xFF, 0xFF, 0xFF,
           0xFF, 0xFF, 0xFF, 0xFF,
           0xFF, 0xFF, 0xFF, 0xFF,
           0xFF, 0xFF, 0xFF, 0xFF,
           0xFF, 0xFF, 0xFF, 0xFF,
           0x00, 0x00, 0xE0, 0x03,
           0x00, 0x00, 0xE0, 0x03  // Code for char 4
           },
           {
           0x00, 0x00, 0x80, 0x03,
           0x00, 0x70, 0x80, 0x0F,
           0x80, 0x7F, 0xC0, 0x1F,
           0xFC, 0xFF, 0xC0, 0x3F,
           0xFF, 0xFF, 0xC0, 0x7F,
           0xFF, 0xFF, 0x00, 0x7F,
           0xFF, 0x7F, 0x00, 0xFC,
           0xFF, 0x38, 0x00, 0xF8,
           0x1F, 0x3C, 0x00, 0xF8,
           0x1F, 0x3E, 0x00, 0xF8,
           0x1F, 0x3E, 0x00, 0xF8,
           0x1F, 0x7E, 0x00, 0xFC,
           0x1F, 0xFE, 0x01, 0x7F,
           0x1F, 0xFC, 0xFF, 0x7F,
           0x1F, 0xF8, 0xFF, 0x3F,
           0x1F, 0xF0, 0xFF, 0x0F,
           0x1F, 0xE0, 0xFF, 0x07,
           0x00, 0x00, 0xFF, 0x00  // Code for char 5
           },
          // {
        //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          // 0x00, 0x00, 0x80, 0x03,
		  // 0x00, 0x70, 0x80, 0x0C,
		  // 0x80, 0x4F, 0x40, 0x10,
		  // 0x7C, 0x80, 0x40, 0x20,
		  // 0x03, 0x80, 0xC0, 0x40,
		  // 0x01, 0x80, 0x00, 0x43,
		  // 0x01, 0x47, 0x00, 0x84,
		  // 0xE1, 0x28, 0x00, 0x88,
		  // 0x11, 0x24, 0x00, 0x88,
		  // 0x11, 0x22, 0x00, 0x88,
		  // 0x11, 0x22, 0x00, 0x88,
		  // 0x11, 0x42, 0x00, 0x84,
		  // 0x11, 0x82, 0x01, 0x43,
		  // 0x11, 0x04, 0xFE, 0x40,
		  // 0x11, 0x08, 0x00, 0x30,
		  // 0x11, 0x10, 0x00, 0x08,
		  // 0x1F, 0xE0, 0x00, 0x07,
		  // 0x00, 0x00, 0xFF, 0x00,  // Code for char 5
        //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          // },
           {
           0x00, 0x00, 0x00, 0x00,
           0x00, 0xFE, 0x7F, 0x00,
           0xC0, 0xFF, 0xFF, 0x03,
           0xF0, 0xFF, 0xFF, 0x0F,
           0xF8, 0xFF, 0xFF, 0x3F,
           0xFC, 0xFF, 0xFF, 0x7F,
           0xFE, 0xE0, 0x03, 0x7F,
           0x3F, 0xF0, 0x01, 0xFC,
           0x1F, 0xF8, 0x00, 0xF8,
           0x1F, 0xF8, 0x00, 0xF8,
           0x1F, 0xF8, 0x00, 0xF8,
           0x7F, 0xF8, 0x01, 0xFC,
           0xFF, 0xF9, 0x03, 0xFE,
           0xFE, 0xF1, 0xFF, 0x7F,
           0xFC, 0xE1, 0xFF, 0x3F,
           0xF8, 0xC0, 0xFF, 0x1F,
           0xC0, 0x80, 0xFF, 0x0F,
           0x00, 0x00, 0xFE, 0x01  // Code for char 6
           },
           {
           0x00, 0x00, 0x00, 0x00,
           0x1F, 0x00, 0x00, 0x00,
           0x1F, 0x00, 0x00, 0x00,
           0x1F, 0x00, 0x00, 0x00,
           0x1F, 0x00, 0x00, 0xFC,
           0x1F, 0x00, 0xE0, 0xFF,
           0x1F, 0x00, 0xFE, 0xFF,
           0x1F, 0xC0, 0xFF, 0xFF,
           0x1F, 0xF0, 0xFF, 0xFF,
           0x1F, 0xFC, 0xFF, 0x03,
           0x1F, 0xFF, 0x1F, 0x00,
           0xDF, 0xFF, 0x01, 0x00,
           0xFF, 0x7F, 0x00, 0x00,
           0xFF, 0x0F, 0x00, 0x00,
           0xFF, 0x03, 0x00, 0x00,
           0xFF, 0x00, 0x00, 0x00,
           0x7F, 0x00, 0x00, 0x00,
           0x1F, 0x00, 0x00, 0x00  // Code for char 7
           },
           {
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0xF8, 0x03,
           0xE0, 0x07, 0xFE, 0x0F,
           0xF8, 0x1F, 0xFF, 0x1F,
           0xFC, 0xBF, 0xFF, 0x3F,
           0xFE, 0xFF, 0xFF, 0x7F,
           0xFE, 0xFF, 0x07, 0x7E,
           0x3F, 0xF8, 0x03, 0xFC,
           0x1F, 0xF0, 0x01, 0xF8,
           0x1F, 0xF0, 0x01, 0xF8,
           0x1F, 0xF0, 0x01, 0xF8,
           0x3F, 0xF8, 0x03, 0xFC,
           0xFF, 0xFF, 0x07, 0xFE,
           0xFE, 0xFF, 0xFF, 0x7F,
           0xFC, 0xBF, 0xFF, 0x3F,
           0xF8, 0x1F, 0xFF, 0x1F,
           0xE0, 0x07, 0xFE, 0x0F,
           0x00, 0x00, 0xF8, 0x03  // Code for char 8
           },
           {
           0x80, 0x7F, 0x00, 0x00,
           0xF0, 0xFF, 0x01, 0x03,
           0xF8, 0xFF, 0x03, 0x1F,
           0xFC, 0xFF, 0x87, 0x3F,
           0xFE, 0xFF, 0x8F, 0x7F,
           0x7E, 0xC0, 0x8F, 0x7F,
           0x3F, 0x80, 0x1F, 0xFE,
           0x1F, 0x00, 0x1F, 0xF8,
           0x1F, 0x00, 0x1F, 0xF8,
           0x1F, 0x00, 0x1F, 0xF8,
           0x1F, 0x00, 0x1F, 0xF8,
           0x3E, 0x80, 0x0F, 0x7C,
           0xFE, 0xC0, 0x07, 0x7F,
           0xFC, 0xFF, 0xFF, 0x3F,
           0xF8, 0xFF, 0xFF, 0x1F,
           0xF0, 0xFF, 0xFF, 0x0F,
           0xC0, 0xFF, 0xFF, 0x03,
           0x00, 0xFE, 0x3F, 0x00   // Code for char 9
           },
           {
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00  // Code for char  (10)
           },
           {
           0x00, 0x00, 0x00, 0x00,
           0xFF, 0x01, 0x00, 0x00,
           0xFF, 0x07, 0x00, 0x00,
           0xFF, 0x07, 0x00, 0x00,
           0xFF, 0x07, 0x00, 0x00,
           0xFF, 0x01, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0xFF, 0x01, 0x00, 0x00,
           0xFF, 0x07, 0x00, 0x00,
           0xFF, 0x07, 0x00, 0x00,
           0xFF, 0x07, 0x00, 0x00,
           0xFF, 0x01, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00  // Code for char " (11)
           }
           };
   //------------------------------------
   //    
   //------------------------------------------
   unsigned char MP [40] = {
	       0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x70,
		   0x00, 0x00, 0x00, 0xf8,
		   0x00, 0x00, 0x00, 0xf8,
		   0x00, 0x00, 0x00, 0xf8,
		   0x00, 0x00, 0x00, 0x70,
		   0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00
           };
 //  unsigned char MD [12] [10] [3] = {
  // unsigned char [ , ,] M0 = {  "0"
	//	   {
	//	   {0xfc, 0xff, 0x3f},  //i=0, j=0, k=0; i=0, j=0, k=1; i=0, j=0, k=2
	//	   {0xfe, 0xff, 0x7f},  //i=0, j=1, k=0; i=0, j=1, k=1; i=0, j=1, k=2
	//	   {0xff, 0xff, 0xff},  //i=0, j=2, k=0; i=0, j=2, k=1; i=0, j=2, k=2
	//	   {0x0f, 0x00, 0xf0},
	//	   {0x07, 0x00, 0xe0},
	//	   {0x07, 0x00, 0xe0},
	//	   {0x0f, 0x00, 0xf0},
	//	   {0xff, 0xff, 0xff},
	//	   {0xfe, 0xff, 0x7f},
	//	   {0xfc, 0xff, 0x3f},  //i=0, j=9, k=0; i=0, j=9, k=1; i=0, j=9, k=2
     //      },//;
   //---------------------------------------
   //   
  unsigned char init[] =
   {
   0xAE, //  

   0xD5, //    
   0x80,
   ///+-----  0-F/ 0 -   1
   //+------  .   0x80
   0xA8, //  multiplex ratio
   0x3F, // 1/64 duty (  ), 0x1F - 128x32, 0x3F - 128x64

   0xD3, //   (offset)
   0x00, //  

   0x40, //     0x40   RAM

   0x8D, //   
   0x14, // 0x10 -  (VCC  ) 0x14 -   DC/DC

   0x20, //   
   0x01, // 0-       ()
         // 1 -       
         // 2 -      

   0xA0, //     ( X)
         // A1 -   (/) A0 -  (/)

   0xC0, //    
         //    
         // 0 - / (   )
         // 8 - / (   )
   0xDA, //   COM
   0x12, // 0x02 - 128x32, 0x12 - 128x64

   0x81, //   () 
   0x8f,//8f, // 0x8F..0xCF (1-256)

   0xD9, //   DC/DC 
   0xF1, // 0x22 - VCC   / 0xF1  

   0xDB, //   VcomH
   0x10, //10 - 3.5 ma, 20 - 3.67 ma, 40-5.53ma     0x00..0x70

   0xA4, //  

   0xA6, // 0xA6 -  , 0xA7 -  

   0xAF  //  
   };
//--------------------------------------------
   unsigned long ADCSR,ksr;
//--------------
       uint8_t SFRPAGE_save;
//--------------------
//   (ushort)    dt:
// u.dt-ushort,u.bdt.dth-.,u.bdt.dtl-..
//------------------------------------------------------
             union {
             unsigned short dt; //us.dt
                struct{
                  unsigned char dth;// us.bdt.dth
                  unsigned char dtl;// us.bdt.dtl
                      }bdt;
                   }us;
//--------------------------------
void outspi(unsigned char byte) {
//----------
	SPI0DAT = byte;        //    SPI
	while (!SPI0CN0_SPIF); //    
	SPI0CN0_SPIF = 0;      //    .
	//DEL20NS();           //      SPIF.
}
//-----------------
void outcmd(unsigned char byte) {
	DC=0;
	CS=0;
//	DEL20NS();
	outspi(byte);
	CS=1;
}
//--------------------------------
void outdat(unsigned char byte) {
	DC=1;
	CS=0;
//	DEL20NS();
	outspi(byte);
	CS=1;
}
//--------------------------------
//unsigned char inspi() {
	//----------
//	unsigned char byte;
//	SPI0DAT = 0x0f;         //    .
//	while (!SPI0CN0_SPIF);  //    
//	SPI0CN0_SPIF = 0;       //    .
//	byte = SPI0DAT;
//	DEL20NS();   //      SPIF.
//	return (byte);
//}
//----------------
//   OLED
//---------------------------------
void CLS (void){
	unsigned short i;
//-----------
//Reset
//-------------
RES=1;
DEL2US();
RES=0;
DEL2US();
RES=1;
DEL2US();
//-------------------
// OLED
//-------------------------
  DC=0; //
  CS=0;
   for(i = 0; i < sizeof (init); i++) {
	outspi(init[i]);   //   
   }
	CS=1;
//-----------------------
//  
//----------------------
	outcmd(0x21); //  
	outcmd(0);    //  
	outcmd(127);  //  

	outcmd(0x22); //  
	outcmd(0);    //  
	outcmd(7);    //  

	DC=1;//
    CS=0;
	for(i = 0; i < 1024; i++) {
		outspi(0);	// 
	}
	CS=1;
}

//-------------------------------
// /  . mesto=0 -  1- , mesto=1 -  2- .
//---------------------
  void out_point (unsigned char mesto){
//void OUTPOINT(unsigned char mesto){
	unsigned char i, adr;

	if (mesto == 0)
		adr=20;
	if (mesto == 1)
		adr=38;

	outcmd(0x21);   //  
	outcmd(adr);    //  
	outcmd(adr+10); //  

	outcmd(0x22);   //  
	outcmd(2);      //  
	outcmd(5);      //  

    DC=1; //data
    CS=0;

	for(i = 0; i < 40; i++) {
		outspi(MP[i]);//   .
	}
	CS=1;
  }
//-------------------------------------
//  /  .
//--------------------
void OUTCIFRA(unsigned char cifra, unsigned char adr){
	unsigned char j;

	outcmd(0x22);   //  
	outcmd(2);      //  
	outcmd(5);      //  

	outcmd(0x21);   //  
	outcmd(adr);    //  
	outcmd(adr+17); //  

    DC=1; //data
    CS=0;

	for(j = 0; j < 72; j++) {
		outspi(MD[cifra][j]);//   .
	}
	CS=1;
   }
//--------------------------------
//  /    . mesto = 0,1,2,3,4,5,10. 1 -   2- , 10 -   0-.
//----------------------------------------------------------------------------------------------
void out_cif_mes (unsigned char cifra, unsigned char mesto){
	unsigned char adr;
   if (mesto == 0)
	   adr = 0;
   if (mesto==1)
       adr = 30;
   if (mesto==10)
       adr = 20;
   if (mesto == 2 || mesto == 3 || mesto == 4 || mesto == 5) //50,70,90,110
	   adr = 50+(mesto-2)*20;
   OUTCIFRA(cifra, adr);
   }
//--------------------------------------
 void StartAutoscan(void);
 void readADC_XRAM(void);
 void SUMMA(void);
 //void adc8(void);
//-----------------------------------


//-----------------------------------------------------------------------------
    void SiLabs_Startup (void)
    {
      // Disable the watchdog here
    }


    // Main Routine
//-----------------------------------------------------------------------------
void main(void)
{
  enter_DefaultMode_from_RESET();
  IE_EA = 0;       // disble global interrupts

 // j = M1[0] [1] [3];
//=====================
   N=0;
	SFRPAGE = 0x00;

  //---------
  //  
  //------------
   //   SFRPAGE   = SPI0_PAGE;
  //------------------
  CLS(); //  

  //------------   -------------

    //OUTCIFRA(5, 110);

  //	out_cif_mes (6, 0); //   "5"  0- 
  	///OUTPOINT(0);        //    1- 
  //	out_point(0);
  //	out_cif_mes (6, 1); //   "4"  1- 
  //	out_cif_mes (6, 2); //   "3"  2- 
  //	out_cif_mes (6, 3); //   "2"  3- 
  //	out_cif_mes (6, 4); //   "1"  4- 
  ///	out_cif_mes (6, 5); //   "0"  5- 



   // out_cif_mes (10, 0); //    (10-)   0- 
   // out_cif_mes (5, 10); //   "5"  2-  (  0- )
   // OUTPOINT(1);         //    2- 
  	//out_point(1);
   // out_cif_mes (3, 2);  //   "3"  2- 
  //	out_cif_mes (2, 3);  //   "2"  3- 
  //	out_cif_mes (1, 4);  //   "1"  4- 
  //	out_cif_mes (0, 5);  //   "0"  5- 

    //	while(1);
//--------------------------------------------------------


  EIE1 = EIE1_EADC0__ENABLED | EIE1_EWADC0__DISABLED | EIE1_ECP0__DISABLED
	  			| EIE1_ECP1__DISABLED | EIE1_EMAT__DISABLED | EIE1_EPCA0__DISABLED
	  			| EIE1_ESMB0__DISABLED | EIE1_ET3__DISABLED;

	  	IE_EA =0;// disable global interrupts


  //-------------------------
  //##########################################################
  if (CAL0 == 0) {

   //   /  -  PWR.F/PRD.F   PWR.B/PRD.B
   //---------------------------------------------------------------------------------


 	  PWR.F = 6.54321;
 	  FLASH_PageErase(START_ADDRESS_P);//  - 512 .
 	  FLASH_Write(START_ADDRESS_P, PWR.B, sizeof(PWR.B));// PWR.F  -.
 	  FLASH_Read(PRD.B, START_ADDRESS_P, sizeof(PRD.B)); // PRD.F  -.

 	  //------------------
 	  //      PRD.F
 	  //---------------------------------------------
 	  sprintf(buf3,"%7.5f",PRD.F);//   PRD.F  .

 	 	        for (i = 0; i<7; i++){
 	 	      	  buf3[i]&=0x0f;
 	 	      	   }

 	 	        buf1[0]=buf3[0];
 	 	        buf1[1]=buf3[2];
 	 	        buf1[2]=buf3[3];

 	 	        buf2[0]=buf3[4];
 	 	        buf2[1]=buf3[5];
 	 	        buf2[2]=buf3[6];

 	      	  //   CSML = 0;
 	      	  //   DEL2US();
 	      	  //   outspi(0x00);//  1- .
 	      	  //   outspi((M[buf1[0]])|0x80); // 2-    (1- ).
 	      	  //   outspi(M[buf1[1]]);        //  3,4 
 	      	  //   outspi(M[buf1[2]]);        // (2,3 - ).
 	      	  //   outspi(0x16);//  4-  -  .
 	      	  //   DEL2US();
 	      	  //   CSML = 1;

 	      	  //   CSST=0;
 	      	  //	 DEL2US();

 	      	  // 	 outspi(M[buf2[0]]);// 5,6,7 
 	      	  //	 outspi(M[buf2[1]]);//(4,5,6 
 	      	  //	 outspi(M[buf2[2]]);// ).

 	      	  // 	 outspi(0x00);//  8 .

 	      	  //	 outspi(0x08);//  .(x8)+  .
 	     	  //   DEL2US();
 	      	  //	 CSST = 1;
 	  //----------------------------------------------
//    "-CAL.0=-"
 	      	  //   CSML = 0;
 	      	  //   DEL2US();
 	      	  //   outspi(0x00);//  1- .
 	      	  //   outspi(0x40);//  "-".
 	      	  //   outspi(0x39);//  "C".
 	      	  //   outspi(0x77);//  "A".
 	      	  //   outspi(0x16);//  4-  -  .
 	      	  //   DEL2US();
 	      	  //   CSML = 1;

 	      	  //   CSST=0;
 	      	  // 	 DEL2US();

 	      	  //	 outspi(0xb8);//  "L.".
 	      	  //	 outspi(0x3f);//  "0".
 	      	  //	 outspi(0x40);//  "-".
 	      	  //	 outspi(0x00);//  8 .

 	      	  //	 outspi(0x08);//  .(x8)+  .
 	     	  //   DEL2US();
 	      	  //	 CSST = 1;

 	      	//	while(1);
//-----------------------------------------
//                "0"  
//-----------------------------------------
      out_cif_mes (11, 2); //   (11-)   2- 
 	  out_cif_mes (0, 3);  //      (0-)    3- 
 	  out_cif_mes (11, 4); //   (11-)   4- 
//-------------------------------------------


	  for (jk = 0; jk<7; jk++){// .
	  	AD0.UL=0;
	  	readADC_XRAM();// AD0.UL    (V0)
        AD0.UL >>= 8;
	  	}

 	  	AD0.UL=0;
 	  	readADC_XRAM();// AD0.UL    (V0).
        AD0.UL >>= 8;


        BWR.F = (1.0*AD0.UL)/524288.0;//21.03.20

 //------------------------------
 //      -.
 // BWR.F - float   , BRD.F float   .
 // BWR.B -  [4]  ,  BRD.B -  [4]  .
 //--------------------------------------------------------

 	  BRD.F = 0xff;
 	  FLASH_PageErase(START_ADDRESS_B);//  - 512 .
 	  FLASH_Write(START_ADDRESS_B, BWR.B, sizeof(BWR.B));// BWR.F  -.
 	  FLASH_Read(BRD.B, START_ADDRESS_B, sizeof(BRD.B)); // BRD.F  -.

 	  AD0.UL=0;
 	  readADC_XRAM();// AD0.UL    (V0)
 	  AD0.UL >>= 8;

 	 for (jk = 0; jk<7; jk++){// .
 	 	  	AD0.UL=0;
 	 	  	readADC_XRAM();// AD0.UL    (V1)
 	         AD0.UL >>= 8;
 	 	  	}

 	  F=(1.0*AD0.UL)/524288.0;//

 	  F=F-BRD.F;

 	  F = 3.0*F;
 	 // F = 2.0*F;

 //-------------------------------------------------------------------------------------
 //     LCD.
 //-----------------------
 	  sprintf(buf3,"%7.5f",F);//    .

 	        for (i = 0; i<7; i++){
 	      	  buf3[i]&=0x0f;
 	      	   }

 	     //   buf1[0]=buf3[0];
 	     //   buf1[1]=buf3[2];
 	     //   buf1[2]=buf3[3];

 	     //   buf2[0]=buf3[4];
 	     //   buf2[1]=buf3[5];
 	     //   buf2[2]=buf3[6];
//-----------------------------------
 	        out_cif_mes (buf3[0], 0); //  1-   0- 
 	        out_point (0);            //    1- 
 	        out_cif_mes (buf3[2], 1); //  2-   1- 
 	        out_cif_mes (buf3[3], 2); //  3-   2- 
 	        out_cif_mes (buf3[4], 3); //  4-   3- 
 	        out_cif_mes (buf3[5], 4); //  3-   4- 
 	        out_cif_mes (buf3[6], 5); //  3-   5- 
//-----------------------------------
      	  //   CSML = 0;
      	  //   DEL2US();
      	  //   outspi(0x00);//  1- .
      	  //   outspi((M[buf1[0]])|0x80); // 2-    (1- ).
      	  //   outspi(M[buf1[1]]);        //  3,4 
      	  //   outspi(M[buf1[2]]);        // (2,3 - ).
      	  //   outspi(0x16);//  4-  -  .
      	  //   DEL2US();
      	  //   CSML = 1;

      	  //   CSST=0;
      	  //	 DEL2US();

      	  //	 outspi(M[buf2[0]]);// 5,6,7 
      	  //	 outspi(M[buf2[1]]);//(4,5,6 
      	  //	 outspi(M[buf2[2]]);// ).

      	  //	 outspi(0x00);//  8 .

      	  //	 outspi(0x08);//  .(x8)+  .
     	  //   DEL2US();
      	  //	 CSST = 1;
 //--------------------------------------

 	while(1);
  }
  //##########################################################
  //----------------------------------------------------------------
  if (CAL1 == 0) {

  //   /  -  PWR.F/PRD.F   PWR.B/PRD.B
  //---------------------------------------------------------------------------------

	 	  PWR.F = 1.23456;
	 	  FLASH_PageErase(START_ADDRESS_P);//  - 512 .
	 	  FLASH_Write(START_ADDRESS_P, PWR.B, sizeof(PWR.B));// PWR.F  -.
	 	  FLASH_Read(PRD.B, START_ADDRESS_P, sizeof(PRD.B)); // PRD.F  -.

  //------------------
  //      PRD.F
  //---------------------------------------------
	 	  sprintf(buf3,"%7.5f",PRD.F);//   PRD.F  .

	 	 	        for (i = 0; i<7; i++){
	 	 	      	  buf3[i]&=0x0f;
	 	 	      	   }

	 	 	        buf1[0]=buf3[0];
	 	 	        buf1[1]=buf3[2];
	 	 	        buf1[2]=buf3[3];

	 	 	        buf2[0]=buf3[4];
	 	 	        buf2[1]=buf3[5];
	 	 	        buf2[2]=buf3[6];

	 	    //  	     CSML = 0;
	 	    //  	     DEL2US();
	 	    //  	     outspi(0x00);//  1- .
	 	    //  	     outspi((M[buf1[0]])|0x80); // 2-    (1- ).
	 	    //  	     outspi(M[buf1[1]]);        //  3,4 
	 	    //  	     outspi(M[buf1[2]]);        // (2,3 - ).
	 	    //  	     outspi(0x16);//  4-  -  .
	 	    //  	     DEL2US();
	 	    //  	     CSML = 1;

	 	    //  	     CSST=0;
	 	    //  		 DEL2US();

	 	    //  		 outspi(M[buf2[0]]);// 5,6,7 
	 	    //  		 outspi(M[buf2[1]]);//(4,5,6 
	 	    //  		 outspi(M[buf2[2]]);// ).

	 	    //  		 outspi(0x00);//  8 .

	 	    //  		 outspi(0x08);//  .(x8)+  .
	 	    // 	     DEL2US();
	 	    //  		 CSST = 1;
//----------------------------------------------
	 	      		//    "-CAL.1=-"
	 	     // 		 	      	     CSML = 0;
	 	     // 		 	      	     DEL2US();
	 	     // 		 	      	     outspi(0x00);//  1- .
	 	     // 		 	      	     outspi(0x40);//  "-".
	 	     // 		 	      	     outspi(0x39);//  "C".
	 	     // 		 	      	     outspi(0x77);//  "A".
	 	     // 		 	      	     outspi(0x26);//  3-  -  ..
	 	     // 		 	      	     DEL2US();
	 	     // 		 	      	     CSML = 1;

	 	     // 		 	      	     CSST=0;
	 	     // 		 	      		 DEL2US();

	 	     // 		 	      		 outspi(0xb8);//  "L.".
	 	     // 		 	      		 outspi(0x06);//  "1".
	 	     // 		 	      		 outspi(0x40);//  "-".
	 	     // 		 	      		 outspi(0x00);//  8 .

             //		 	      		 outspi(0x08);//  .(x8)+  .
	 	     // 		 	     	     DEL2US();
	 	     ///// 		 	      		 CSST = 1;

	 	      		 	     //	while(1);
//-----------------------------------------
//                "3"  
//-----------------------------------------
        out_cif_mes (11, 2); //   (11-)   2- 
        out_cif_mes (3, 3);  //      (3-)    3- 
	 	out_cif_mes (11, 4); //   (11-)   4- 
//--------------------------------------------

	  	FLASH_Read(BRD.B, START_ADDRESS_B, sizeof(BRD.B)); // BRD.F  - (V0).

	   	for (jk = 0; jk<7; jk++){// .
	  	AD0.UL=0;
	  	readADC_XRAM();// AD0.UL    (V1)
        AD0.UL >>= 8;
	  	}
	  	//ksr=0;
	  	for (jk = 0; jk<7; jk++){// 8 .
	  		  	AD0.UL=0;
	  		  	readADC_XRAM();// AD0.UL    (V1)
	  	        AD0.UL >>= 8;
	  	         	}

	   	KWR.F = (1.0*AD0.UL)/524288.0; // V1.

        KWR.F = 1.0/(KWR.F-BRD.F); // K=1/K=1/(V1-V0).

//------------------------------
//      -.
// KWR.F - float   , KRD.F float   .
// KWR.B -  [4]  ,  KRD.B -  [4]  .
//--------------------------------------------------------

   	  KRD.F = 0;
	  FLASH_PageErase(START_ADDRESS_K);//  - 512 .
	  FLASH_Write(START_ADDRESS_K, KWR.B, sizeof(KWR.B));// KWR.F  -.
	  FLASH_Read(KRD.B, START_ADDRESS_K, sizeof(KRD.B)); // KRD.F  -.

	  AD0.UL=0;
	  readADC_XRAM();// AD0.UL   
	  AD0.UL >>= 8;

	  F=(1.0*AD0.UL)/524288.0;//    3   .//21.03.20

	  F=KRD.F*(F-BRD.F);// V=K(V-V0).
	  F=F*3.0;//
//-------------------------------------------------------------------------------------
//  3    LCD.
//-----------------------
	  sprintf(buf3,"%7.5f",F);

	        for (i = 0; i<7; i++){
	      	  buf3[i]&=0x0f;
	      	   }

	     //   buf1[0]=buf3[0];
	     //   buf1[1]=buf3[2];
	     //   buf1[2]=buf3[3];

	     //   buf2[0]=buf3[4];
	     //   buf2[1]=buf3[5];
	     //   buf2[2]=buf3[6];
//-----------------------------------
            out_cif_mes (buf3[0], 0); //  1-   0- 
	        out_point (0);            //    1- 
	        out_cif_mes (buf3[2], 1); //  2-   1- 
	        out_cif_mes (buf3[3], 2); //  3-   2- 
	        out_cif_mes (buf3[4], 3); //  4-   3- 
	        out_cif_mes (buf3[5], 4); //  3-   4- 
	        out_cif_mes (buf3[6], 5); //  3-   5- 
//-----------------------------------

     	  //   CSML = 0;
     	  //   DEL2US();
     	  //   outspi(0x00);//  1- .
     	  //   outspi((M[buf1[0]])|0x80); // 2-    (1- ).
     	  //   outspi(M[buf1[1]]);        //  3,4 
     	  //   outspi(M[buf1[2]]);        // (2,3 - ).
     	  //   outspi(0x26);//  3-  -   .
     	  //   DEL2US();
     	  //   CSML = 1;

     	  //   CSST=0;
     	  //	 DEL2US();

     	  //	 outspi(M[buf2[0]]);// 5,6,7 
     	  //	 outspi(M[buf2[1]]);//(4,5,6 
     	  //	 outspi(M[buf2[2]]);// ).

     	  //	 outspi(0x00);//  8 .

     	  //	 outspi(0x08);//  .(x8)+  .
    	  //   DEL2US();
     	  //	 CSST = 1;
//--------------------------------------

	while(1);
//-------------
  }
//-----------------------------
	FLASH_Read(KRD.B, START_ADDRESS_K, sizeof(KRD.B)); // KRD.F  -.
	FLASH_Read(BRD.B, START_ADDRESS_B, sizeof(BRD.B)); // BRD.F  - (V0).


	START:

	  AD0.UL=0;
      readADC_XRAM();// AD0.UL   
      AD0.UL >>= 8;

//----------------------------
	  F=(1.0*AD0.UL)/524288.0;//    3   .
//----------------------------------------------
      F=F-BRD.F;  //1
      F=F*KRD.F;  //2
    //  F=3.0*KRD.F*(F-BRD.F);// V=K(V-V0).
      F=3.0*F;   //4

      if (DIAP==0){

         F=2.0*F;    //3

         sprintf(buf3,"%7.5f",F);

         for (i = 0; i<7; i++){
          	  buf3[i]&=0x0f;
          	   }

      out_cif_mes (buf3[0], 0);   //  1-   0- 
      out_point (0);              //    1- 
      out_cif_mes (buf3[2], 1);   //  2-   1-  buf3[1] , ..   
      out_cif_mes (buf3[3], 2);   //  3-   2- 
     }
      else {

         F=20.0*F;//3

    	 sprintf(buf3,"%7.4f",F);

    	for (i = 0; i<7; i++){
    	      buf3[i]&=0x0f;
          }
           //------------   "0"  ------
    		if (buf3[0]==0)
       		MB0=10;                  //  
    		else
      		MB0=buf3[0];
          //------------------------------------------------
      out_cif_mes (MB0, 0);          //  1-      0- 
      out_cif_mes (buf3[1], 10);      //  2-   1- 
      out_point (1);                 //    2- 
	  out_cif_mes (buf3[3], 2);      //  3-   2- 
    }
//--------------------------------
//------------ 4, 5  6      ------------
      out_cif_mes (buf3[4], 3);      //  4-   3- 
      out_cif_mes (buf3[5], 4);      //  5-   4- 
      out_cif_mes (buf3[6], 5);      //  6-   5- 
//-------------------------------------------------------//01.12.19

  		goto START;


}   //  main
//-----------------------------------------------------------------------------
// StartAutoscan
//-----------------------------------------------------------------------------
//
// Starts a new Autoscan to fill up BUFFER. Clears all global variables to
// to their original states and begins the first scan. Subsequent scans
// are handled in the ADC ISR.
//
//--------------------------------------
  void StartAutoscan(void)
{
  BUFFER_FULL = false;
  BUFFER_INDEX = 0;
  // The first scan will begin at the start of BUFFER.
  ADC0_enableAutoscan(&BUFFER[BUFFER_INDEX], SAMPLES_PER_SCAN, ADC0_AUTOSCAN_MODE_CONTINUOUS);

  // The scan that will start after this one completes will begin SAMPLES_PER_SCAN ahead
  // of the first.
  BUFFER_INDEX += SAMPLES_PER_SCAN;
  ADC0_setNextAutoscanBuffer(&BUFFER[BUFFER_INDEX], SAMPLES_PER_SCAN);

  // Start the first scan.
  ADC0_startConversion();
}


//-----------------------------------------------------------------------------
// FindMinMaxAverage
//-----------------------------------------------------------------------------
//
// Finds the minimum, maximum, and average value for the values within
// BUFFER.
//
void SUMMA(void)
{
 uint32_t sum = 0;

 uint16_t i;
 uint16_t sample;
 for (i = 0; i < BUFFER_SIZE; i++)
  {
   sample = BUFFER[i];

   sum += sample;
 }
 AD0.UL = sum;
}

//-----------------------------------------------------------------------------

bool ADC0_isAutoscanEnabled(void)
{
  return ((ADC0ASCF & ADC0ASCF_ASEN__BMASK) == ADC0ASCF_ASEN__START_SCAN);
}

//################################
   void readADC_XRAM()
   {
	   dt=64;//128;//64;//32;//16;
  //      SFRPAGE = DMA0_PAGE;
   //--------
     switch (dt) {
       case 1:       //dt=1-0.05c(20 ),2-0.1c(10 ),5-0.5c(2 ),10-1c(1 Hz)
            l=1;
            sdv=2;
       break;

       case 2:
            l=2;
            sdv=1;
       break;

       case 4:
            l=4;
            sdv=0;
       break;

       case 8:
            l=8;
            sdv=1;
       break;

       case 16:
            l=16;
            sdv=1; //1-BUFFER_SIZE (1024),2-BUFFER_SIZE (2048)
       break;

       case 32:
            l=32;
            sdv=2;//2-BUFFER_SIZE (1024),3-BUFFER_SIZE (2048)
       break;

       case 64:
                   l=64;//^ 5   . -     
                   sdv=3;//3-BUFFER_SIZE (1024),4-BUFFER_SIZE (2048)
              break;

       case 128:
                          l=128;//  ,    ~ 2   .
                          sdv=4;//4-BUFFER_SIZE (1024),5-BUFFER_SIZE (2048)
                     break;
       default:
           l=1;
           sdv=2;
       break;

       } //  switch
   //--------
        ADCSR=0;

        IE_EA = 1;// enable global interrupts

        for (k=0;k<l;k++) {

//************************************
            StartAutoscan();
            while (BUFFER_FULL);
            SUMMA();// AD0.UL
//***************************************
       //ADCSR=ADCSR+AD0.UL;
       ADCSR+=AD0.UL;
         } //  for

        IE_EA =0;// disable global interrupts

        if (l<8)
        {
        ADCSR=ADCSR << sdv;
        }
        else
        {
        //ADCSR=ADCSR >> sdv;
        ADCSR>>=sdv;
         }
   	       AD0.UL=ADCSR;

   } //  void read_XRAM()}

//######################################################
//-----------------------------------------------------------------------------
// ADC0EOC_ISR
//-----------------------------------------------------------------------------
// ADC0 End of Conversion interrupt function
//
// Called once an ADC0 conversion interrupt fires. In this example, this
// occurs when an Autoscan is complete.
//
//-----------------------------------------------------------------------------
SI_INTERRUPT (ADC0EOC_ISR, ADC0EOC_IRQn)
{
  static bool lastScan = false;
  ADC0_clearIntFlags(ADC0_CONVERSION_COMPLETE_IF);

  // If this completed scan was the last one, set the BUFFER_FULL flag
  // and return to let main() process the buffer.
  if (lastScan)
  {
    lastScan = false;
    BUFFER_FULL = true;
    return;
  }

  // Start the next scan
  ADC0_startConversion();

  // If the buffer isn't full, set the next autoscan to fill up the
  // next SAMPLES_PER_SCAN of the buffer. If the current scan that
  // is being executed is the last, disable Autoscan so that it
  // will be disabled after the last scan completes.
  BUFFER_INDEX += SAMPLES_PER_SCAN;
  if (BUFFER_INDEX < BUFFER_SIZE)
  {
    ADC0_setNextAutoscanBuffer(&BUFFER[BUFFER_INDEX], SAMPLES_PER_SCAN);
  }
  else
  {
    ADC0_disableAutoscan();
    lastScan = true;
  }
}
//-----------------------------------------------------------------------------
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
